MODULE_NAME='Rollease Acmeda ARC2 UI v1_1' (DEV dvPanel, DEV vdvDevice, DEV vdvMotors[])

/////////////////////////////////////////////////////////////
// IN ORDER TO VIEW THIS FILE CORRECTLY,
// PLEASE SET YOUR NETLINX STUDIO EDITOR SETTINGS TO:
// TAB STOP CHARACTERS: 2
// INDENT BEFORE TEXT: 2
/////////////////////////////////////////////////////////////

#IF_NOT_DEFINED __SNAPI_CONST__
#INCLUDE 'SNAPI'
#END_IF

(***********************************************************)
(*          DEVICE NUMBER DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_DEVICE

(***********************************************************)
(*               CONSTANT DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_CONSTANT

/////////////////////////////////////////////////////////////
// MAX
/////////////////////////////////////////////////////////////

MAX_STRING_LENGTH						= 255;
MAX_STEP										= 10;
MAX_MOTORS									= 5;

/////////////////////////////////////////////////////////////
// MAIN
/////////////////////////////////////////////////////////////

BTN_START										= 1;
BTN_HOME										= 2;
BTN_MENU_HUB								= 5;
BTN_MENU_MOTOR							= 6;

/////////////////////////////////////////////////////////////
// HUB
/////////////////////////////////////////////////////////////

BTN_HUB_COMMUNICATING 			= 21;
BTN_HUB_INITIALIZED 				= 22;
BTN_HUB_INITIALIZE 					= 23;
BTN_HUB_DEBUG 							= 24;
BTN_HUB_SETUP 							= 25;

TXT_HUB_MOTOR_ADDRESS 			= 21;
TXT_HUB_NAME 								= 31;
TXT_HUB_MAC 								= 32;

/////////////////////////////////////////////////////////////
// MOTOR
/////////////////////////////////////////////////////////////

BTN_MOTOR_COMMUNICATING 		= 31;
BTN_MOTOR_INITIALIZED 			= 32;

TXT_MOTOR_LABEL 						= 1;
TXT_MOTOR_NAME 							= 5;
TXT_MOTOR_ROOM 							= 6;
TXT_MOTOR_VERSION 					= 11;
TXT_MOTOR_TYPE 							= 12;
TXT_MOTOR_VOLTAGE 					= 13;

BTN_MOTOR_MOVE_UP 					= 41;
BTN_MOTOR_MOVE_DOWN 				= 42;
BTN_MOTOR_JOG_UP 						= 51;
BTN_MOTOR_JOG_DOWN 					= 52;
BTN_MOTOR_POSITION_UP 			= 61;
BTN_MOTOR_POSITION_DOWN 		= 62;
BTN_MOTOR_POSITION_SET 			= 63;
BTN_MOTOR_ROTATION_UP 			= 71;
BTN_MOTOR_ROTATION_DOWN 		= 72;
BTN_MOTOR_ROTATION_SET 			= 73;

LVL_MOTOR_POSITION_SET 			= 1;
LVL_MOTOR_POSITION_CURRENT 	= 2;
LVL_MOTOR_ROTATION_SET 			= 11;
LVL_MOTOR_ROTATION_CURRENT 	= 12;

/////////////////////////////////////////////////////////////
// PAGES
/////////////////////////////////////////////////////////////

PAGE_OPENING[20] 						= 'Opening';
PAGE_HUB[20] 								= 'Command_Processor';
PAGE_MOTOR[20] 							= 'Motor_Control';

/////////////////////////////////////////////////////////////
// FEEDBACK
/////////////////////////////////////////////////////////////

TL_FEEDBACK 								= 1;
LONG FEEDBACK_TIMES[] 			= {300};

/////////////////////////////////////////////////////////////
// CUSTOM MODULE CHANNELS
/////////////////////////////////////////////////////////////

CHAN_DEBUG_TOGGLE						= 301;
CHAN_DEBUG_STATE						= 302;

CHAN_SETUP_TOGGLE						= 303;
CHAN_SETUP_STATE						= 304;

CHAN_MOTOR_COMMUNICATING		= 311;
CHAN_MOTOR_INITIALIZED			= 312;

CHAN_MOTOR_JOG_UP						= 321;
CHAN_MOTOR_JOG_DOWN					= 322;

CHAN_MOTOR_MOVE_UP					= 323;
CHAN_MOTOR_MOVE_DOWN				= 324;
CHAN_MOTOR_MOVE_STOP				= 325;

/////////////////////////////////////////////////////////////
// CUSTOM MODULE LEVELS
/////////////////////////////////////////////////////////////

LVL_CURRENT_POSITION				= 7;
LVL_CURRENT_ROTATION				= 8;

(***********************************************************)
(*              DATA TYPE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_TYPE

STRUCTURE _MOTOR
{
	CHAR sName[MAX_STRING_LENGTH];
	CHAR sRoom[MAX_STRING_LENGTH];
	CHAR sVersion[MAX_STRING_LENGTH];
	CHAR sType[MAX_STRING_LENGTH];
	CHAR sVoltage[MAX_STRING_LENGTH];
	INTEGER nCurrentPosition;
	INTEGER nCurrentRotation;
	INTEGER nLastSetPosition;
	INTEGER nLastSetRotation;
}

(***********************************************************)
(*               VARIABLE DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_VARIABLE

/////////////////////////////////////////////////////////////
// HUB
/////////////////////////////////////////////////////////////

VOLATILE CHAR bDebug;
VOLATILE CHAR bSetup;

/////////////////////////////////////////////////////////////
// MOTOR
/////////////////////////////////////////////////////////////

VOLATILE INTEGER nMotorBtns[] = 
{
	11,
	12,
	13,
	14,
	15
};

VOLATILE INTEGER nCurrentMotor;
VOLATILE _MOTOR uMotor[MAX_MOTORS];
VOLATILE CHAR bPreventUpdate;

(***********************************************************)
(*               LATCHING DEFINITIONS GO BELOW             *)
(***********************************************************)
DEFINE_LATCHING

(***********************************************************)
(*       MUTUALLY EXCLUSIVE DEFINITIONS GO BELOW           *)
(***********************************************************)
DEFINE_MUTUALLY_EXCLUSIVE

(***********************************************************)
(*        SUBROUTINE/FUNCTION DEFINITIONS GO BELOW         *)
(***********************************************************)

/////////////////////////////////////////////////////////////
// UI
/////////////////////////////////////////////////////////////

DEFINE_FUNCTION SEND_TEXT(INTEGER btn, CHAR text[])
{
	SEND_COMMAND dvPanel, "'^TXT-', ITOA(btn), ',0,', text";
}

DEFINE_FUNCTION SHOW_PAGE(CHAR page[])
{
	SEND_COMMAND dvPanel, "'PAGE-', page";
}

DEFINE_FUNCTION UPDATE_LEVEL(INTEGER lvl, INTEGER value)
{
	SEND_LEVEL dvPanel, lvl, value;
}

/////////////////////////////////////////////////////////////
// MOTOR
/////////////////////////////////////////////////////////////

DEFINE_FUNCTION SELECT_MOTOR(INTEGER motor)
{
	IF (motor > 0 && motor <= MAX_MOTORS)
	{
		nCurrentMotor = motor;
		POPULATE_MOTOR_INFO(motor);
	}
}

DEFINE_FUNCTION POPULATE_MOTOR_INFO(INTEGER motor)
{
	IF (motor > 0 && motor <= MAX_MOTORS)
	{
		SEND_TEXT(TXT_MOTOR_LABEL, "'MOTOR ', ITOA(motor)");
		SEND_TEXT(TXT_MOTOR_NAME, uMotor[motor].sName);
		SEND_TEXT(TXT_MOTOR_ROOM, uMotor[motor].sRoom);
		SEND_TEXT(TXT_MOTOR_VERSION, uMotor[motor].sVersion);
		SEND_TEXT(TXT_MOTOR_TYPE, uMotor[motor].sType);
		SEND_TEXT(TXT_MOTOR_VOLTAGE, uMotor[motor].sVoltage);
		
		UPDATE_LEVEL(LVL_MOTOR_POSITION_CURRENT, uMotor[motor].nCurrentPosition);
		UPDATE_LEVEL(LVL_MOTOR_ROTATION_CURRENT, uMotor[motor].nCurrentRotation);
		UPDATE_LEVEL(LVL_MOTOR_POSITION_SET, uMotor[motor].nLastSetPosition);
		UPDATE_LEVEL(LVL_MOTOR_ROTATION_SET, uMotor[motor].nLastSetRotation);
	}
}

(***********************************************************)
(*                STARTUP CODE GOES BELOW                  *)
(***********************************************************)
DEFINE_START

TIMELINE_CREATE(TL_FEEDBACK, FEEDBACK_TIMES, 1, TIMELINE_ABSOLUTE, TIMELINE_REPEAT);

(***********************************************************)
(*                THE EVENTS GO BELOW                      *)
(***********************************************************)
DEFINE_EVENT

/////////////////////////////////////////////////////////////
// PANEL ONLINE
/////////////////////////////////////////////////////////////

DATA_EVENT[dvPanel]
{
	ONLINE:
	{
		SHOW_PAGE(PAGE_OPENING);
	}
}

/////////////////////////////////////////////////////////////
// PAGE SELECT
/////////////////////////////////////////////////////////////

BUTTON_EVENT[dvPanel, BTN_START]
{
	PUSH:
	{
		SHOW_PAGE(PAGE_HUB);
	}
}

BUTTON_EVENT[dvPanel, BTN_HOME]
{
	PUSH:
	{
		SHOW_PAGE(PAGE_OPENING);
	}
}

BUTTON_EVENT[dvPanel, BTN_MENU_HUB]
{
	PUSH:
	{
		SHOW_PAGE(PAGE_HUB);
	}
}

BUTTON_EVENT[dvPanel, BTN_MENU_MOTOR]
{
	PUSH:
	{
		SELECT_MOTOR(1);
		SHOW_PAGE(PAGE_MOTOR);
	}
}

/////////////////////////////////////////////////////////////
// MOTOR MENU
/////////////////////////////////////////////////////////////

BUTTON_EVENT[dvPanel, nMotorBtns]
{
	PUSH:
	{
		STACK_VAR INTEGER nIndex;
		nIndex = GET_LAST(nMotorBtns);
		
		SELECT_MOTOR(nIndex);
	}
}

TIMELINE_EVENT[TL_FEEDBACK]
{
	STACK_VAR INTEGER i;
	
	FOR (i = 1; i <= LENGTH_ARRAY(nMotorBtns); i++)
	{
		[dvPanel, nMotorBtns[i]] = (nCurrentMotor == i);
	}
}

/////////////////////////////////////////////////////////////
// HUB
/////////////////////////////////////////////////////////////

BUTTON_EVENT[dvPanel, BTN_HUB_INITIALIZE]
{
	PUSH:
	{
		TO[dvPanel, BTN_HUB_INITIALIZE];
		SEND_COMMAND vdvDevice,"'REINIT'";
	}
}

TIMELINE_EVENT[TL_FEEDBACK]
{
	[dvPanel, BTN_HUB_COMMUNICATING] = [vdvDevice, DEVICE_COMMUNICATING];
	[dvPanel, BTN_HUB_INITIALIZED] 	 = [vdvDevice, DATA_INITIALIZED];
}

BUTTON_EVENT[dvPanel, BTN_HUB_DEBUG]
{
	PUSH:
	{
		PULSE[vdvDevice, CHAN_DEBUG_TOGGLE];
	}
}

TIMELINE_EVENT[TL_FEEDBACK]
{
	[dvPanel, BTN_HUB_DEBUG] = [vdvDevice, CHAN_DEBUG_STATE];
}

BUTTON_EVENT[dvPanel, BTN_HUB_SETUP]
{
	PUSH:
	{
		PULSE[vdvDevice, CHAN_SETUP_TOGGLE];
	}
}

TIMELINE_EVENT[TL_FEEDBACK]
{
	[dvPanel, BTN_HUB_SETUP] = [vdvDevice, CHAN_SETUP_STATE];
}

DATA_EVENT[vdvDevice]
{
	COMMAND:
	{
		STACK_VAR CHAR sCmd[1024];
		STACK_VAR CHAR sHeader[255];
		STACK_VAR CHAR sValue[255];
		
		sCmd = DATA.TEXT;
		sHeader = DuetParseCmdHeader(sCmd);
		sValue = DuetParseCmdParam(sCmd);
		
		SELECT
		{
			ACTIVE(FIND_STRING(sHeader, "'HUB_NAME'", 1) > 0):
			{
				SEND_TEXT(TXT_HUB_NAME, "sValue");
			}
			
			ACTIVE(FIND_STRING(sHeader, "'HUB_MAC'", 1) > 0):
			{
				SEND_TEXT(TXT_HUB_MAC, "sValue");
			}
			
			ACTIVE(FIND_STRING(sHeader, "'MOTOR_ADDRESS'", 1) > 0):
			{
				SEND_TEXT(TXT_HUB_MOTOR_ADDRESS, "sValue");
			}
		}
	}
}

/////////////////////////////////////////////////////////////
// MOTORS
/////////////////////////////////////////////////////////////

BUTTON_EVENT[dvPanel, BTN_MOTOR_MOVE_UP]
{
	PUSH:
	{
		TO[dvPanel, BTN_MOTOR_MOVE_UP];
		
		IF (nCurrentMotor)
			PULSE[vdvMotors[nCurrentMotor], CHAN_MOTOR_MOVE_UP];
	}
	RELEASE:
	{
		IF (nCurrentMotor)
			PULSE[vdvMotors[nCurrentMotor], CHAN_MOTOR_MOVE_STOP];
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_MOVE_DOWN]
{
	PUSH:
	{
		TO[dvPanel, BTN_MOTOR_MOVE_DOWN];
		
		IF (nCurrentMotor)
			PULSE[vdvMotors[nCurrentMotor], CHAN_MOTOR_MOVE_DOWN];
	}
	RELEASE:
	{
		IF (nCurrentMotor)
			PULSE[vdvMotors[nCurrentMotor], CHAN_MOTOR_MOVE_STOP];
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_JOG_UP]
{
	PUSH:
	{
		TO[dvPanel, BTN_MOTOR_JOG_UP];
		
		IF (nCurrentMotor)
			PULSE[vdvMotors[nCurrentMotor], CHAN_MOTOR_JOG_UP];
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_JOG_DOWN]
{
	PUSH:
	{
		TO[dvPanel, BTN_MOTOR_JOG_DOWN];
		
		IF (nCurrentMotor)
			PULSE[vdvMotors[nCurrentMotor], CHAN_MOTOR_JOG_DOWN];
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_POSITION_UP]
{
	PUSH:
	{
		STACK_VAR INTEGER nCurrent;
		
		TO[dvPanel, BTN_MOTOR_POSITION_UP];
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetPosition;
			
			IF (nCurrent <= 255 - MAX_STEP)
				uMotor[nCurrentMotor].nLastSetPosition = nCurrent + MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetPosition = 255;
				
			UPDATE_LEVEL(LVL_MOTOR_POSITION_SET, uMotor[nCurrentMotor].nLastSetPosition);
		}
	}
	HOLD[2, REPEAT]:
	{
		STACK_VAR INTEGER nCurrent;
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetPosition;
			
			IF (nCurrent <= 255 - MAX_STEP)
				uMotor[nCurrentMotor].nLastSetPosition = nCurrent + MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetPosition = 255;
				
			UPDATE_LEVEL(LVL_MOTOR_POSITION_SET, uMotor[nCurrentMotor].nLastSetPosition);
		}
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_POSITION_DOWN]
{
	PUSH:
	{
		STACK_VAR INTEGER nCurrent;
		
		TO[dvPanel, BTN_MOTOR_POSITION_DOWN];
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetPosition;
			
			IF (nCurrent >= MAX_STEP)
				uMotor[nCurrentMotor].nLastSetPosition = nCurrent - MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetPosition = 0;
				
			UPDATE_LEVEL(LVL_MOTOR_POSITION_SET, uMotor[nCurrentMotor].nLastSetPosition);
		}
	}
	HOLD[2, REPEAT]:
	{
		STACK_VAR INTEGER nCurrent;
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetPosition;
			
			IF (nCurrent >= MAX_STEP)
				uMotor[nCurrentMotor].nLastSetPosition = nCurrent - MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetPosition = 0;
				
			UPDATE_LEVEL(LVL_MOTOR_POSITION_SET, uMotor[nCurrentMotor].nLastSetPosition);
		}
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_POSITION_SET]
{
	PUSH:
	{
		TO[dvPanel, BTN_MOTOR_POSITION_SET];
		
		IF (nCurrentMotor)
		{
			bPreventUpdate = TRUE;
			SEND_LEVEL vdvMotors[nCurrentMotor], LVL_CURRENT_POSITION, uMotor[nCurrentMotor].nLastSetPosition;
			
			WAIT 10
				bPreventUpdate = FALSE;
		}
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_ROTATION_UP]
{
	PUSH:
	{
		STACK_VAR INTEGER nCurrent;
		
		TO[dvPanel, BTN_MOTOR_ROTATION_UP];
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetRotation;
			
			IF (nCurrent <= 255 - MAX_STEP)
				uMotor[nCurrentMotor].nLastSetRotation = nCurrent + MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetRotation = 255;
				
			UPDATE_LEVEL(LVL_MOTOR_ROTATION_SET, uMotor[nCurrentMotor].nLastSetRotation);
		}
	}
	HOLD[2, REPEAT]:
	{
		STACK_VAR INTEGER nCurrent;
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetRotation;
			
			IF (nCurrent <= 255 - MAX_STEP)
				uMotor[nCurrentMotor].nLastSetRotation = nCurrent + MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetRotation = 255;
				
			UPDATE_LEVEL(LVL_MOTOR_ROTATION_SET, uMotor[nCurrentMotor].nLastSetRotation);
		}
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_ROTATION_DOWN]
{
	PUSH:
	{
		STACK_VAR INTEGER nCurrent;
		
		TO[dvPanel, BTN_MOTOR_ROTATION_DOWN];
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetRotation;
			
			IF (nCurrent >= MAX_STEP)
				uMotor[nCurrentMotor].nLastSetRotation = nCurrent - MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetRotation = 0;
				
			UPDATE_LEVEL(LVL_MOTOR_ROTATION_SET, uMotor[nCurrentMotor].nLastSetRotation);
		}
	}
	HOLD[2, REPEAT]:
	{
		STACK_VAR INTEGER nCurrent;
		
		IF (nCurrentMotor)
		{
			nCurrent = uMotor[nCurrentMotor].nLastSetRotation;
			
			IF (nCurrent >= MAX_STEP)
				uMotor[nCurrentMotor].nLastSetRotation = nCurrent - MAX_STEP;
			ELSE
				uMotor[nCurrentMotor].nLastSetRotation = 0;
				
			UPDATE_LEVEL(LVL_MOTOR_ROTATION_SET, uMotor[nCurrentMotor].nLastSetRotation);
		}
	}
}

BUTTON_EVENT[dvPanel, BTN_MOTOR_ROTATION_SET]
{
	PUSH:
	{
		TO[dvPanel, BTN_MOTOR_ROTATION_SET];
		
		IF (nCurrentMotor)
		{
			bPreventUpdate = TRUE;
			SEND_LEVEL vdvMotors[nCurrentMotor], LVL_CURRENT_ROTATION, uMotor[nCurrentMotor].nLastSetRotation;
			
			WAIT 10
				bPreventUpdate = FALSE;
		}
	}
}

TIMELINE_EVENT[TL_FEEDBACK]
{
	IF (nCurrentMotor)
	{
		[dvPanel, BTN_MOTOR_COMMUNICATING] = [vdvMotors[nCurrentMotor], CHAN_MOTOR_COMMUNICATING];
		[dvPanel, BTN_MOTOR_INITIALIZED] 	 = [vdvMotors[nCurrentMotor], CHAN_MOTOR_INITIALIZED];
	}
}

DATA_EVENT[vdvMotors]
{
	COMMAND:
	{
		STACK_VAR INTEGER nIndex;
		STACK_VAR CHAR sCmd[1024];
		STACK_VAR CHAR sHeader[255];
		STACK_VAR CHAR sValue[255];
		
		nIndex = DATA.DEVICE.PORT;
		
		IF (nIndex > 0 && nIndex <= MAX_MOTORS)
		{
			sCmd = DATA.TEXT;
			sHeader = DuetParseCmdHeader(sCmd);
			sValue = DuetParseCmdParam(sCmd);
			
			SELECT
			{
				ACTIVE(FIND_STRING(sHeader, "'MOTOR_NAME'", 1) > 0):
				{
					uMotor[nIndex].sName = sValue;
					
					IF (nIndex == nCurrentMotor)
						SEND_TEXT(TXT_MOTOR_NAME, "sValue");
				}
				
				ACTIVE(FIND_STRING(sHeader, "'MOTOR_ROOM'", 1) > 0):
				{
					uMotor[nIndex].sRoom = sValue;
					
					IF (nIndex == nCurrentMotor)
						SEND_TEXT(TXT_MOTOR_ROOM, "sValue");
				}
				
				ACTIVE(FIND_STRING(sHeader, "'MOTOR_VERSION'", 1) > 0):
				{
					uMotor[nIndex].sVersion = sValue;
					
					IF (nIndex == nCurrentMotor)
						SEND_TEXT(TXT_MOTOR_VERSION, "sValue");
				}
				
				ACTIVE(FIND_STRING(sHeader, "'MOTOR_TYPE'", 1) > 0):
				{
					uMotor[nIndex].sType = sValue;
					
					IF (nIndex == nCurrentMotor)
						SEND_TEXT(TXT_MOTOR_TYPE, "sValue");
				}
				
				ACTIVE(FIND_STRING(sHeader, "'MOTOR_VOLTAGE'", 1) > 0):
				{
					uMotor[nIndex].sVoltage = sValue;
					
					IF (nIndex == nCurrentMotor)
						SEND_TEXT(TXT_MOTOR_VOLTAGE, "sValue");
				}
			}
		}
	}
}

LEVEL_EVENT[vdvMotors, LVL_CURRENT_POSITION]
{
	STACK_VAR INTEGER nIndex;
	
	nIndex = LEVEL.SOURCEDEV.PORT
		
	IF (nIndex > 0 && nIndex <= MAX_MOTORS)
	{
		uMotor[nIndex].nCurrentPosition = LEVEL.VALUE;
		
		IF (!bPreventUpdate && nIndex == nCurrentMotor)
			UPDATE_LEVEL(LVL_MOTOR_POSITION_CURRENT, LEVEL.VALUE);
	}
}

LEVEL_EVENT[vdvMotors, LVL_CURRENT_ROTATION]
{
	STACK_VAR INTEGER nIndex;
	
	nIndex = LEVEL.SOURCEDEV.PORT
		
	IF (nIndex > 0 && nIndex <= MAX_MOTORS)
	{
		uMotor[nIndex].nCurrentRotation = LEVEL.VALUE;
		
		IF (!bPreventUpdate && nIndex == nCurrentMotor)
			UPDATE_LEVEL(LVL_MOTOR_ROTATION_CURRENT, LEVEL.VALUE);
	}
}

DEFINE_PROGRAM
